import { getComplexity } from 'graphql-query-complexity';
import { GraphQLError, separateOperations } from 'graphql';

const createComplexityPlugin = ({ schema, maximumComplexity, estimators, onComplete, createError = (max, actual) => new GraphQLError(`Query too complex. Value of ${actual} is over the maximum ${max}.`), }) => {
    return {
        async requestDidStart() {
            return ({
                async didResolveOperation({ request, document }) {
                    const query = request.operationName
                        ? separateOperations(document)[request.operationName]
                        : document;
                    const complexity = getComplexity({
                        schema,
                        query,
                        variables: request.variables,
                        estimators,
                    });
                    if (complexity >= maximumComplexity) {
                        throw await createError(maximumComplexity, complexity);
                    }
                    if (onComplete) {
                        await onComplete(complexity);
                    }
                },
            });
        },
    };
};

export { createComplexityPlugin };
