(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('graphql-query-complexity'), require('graphql')) :
    typeof define === 'function' && define.amd ? define(['exports', 'graphql-query-complexity', 'graphql'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.streamingIterables = {}, global.graphqlQueryComplexity, global.graphql));
}(this, (function (exports, graphqlQueryComplexity, graphql) { 'use strict';

    const createComplexityPlugin = ({ schema, maximumComplexity, estimators, onComplete, createError = (max, actual) => new graphql.GraphQLError(`Query too complex. Value of ${actual} is over the maximum ${max}.`), }) => {
        return {
            async requestDidStart() {
                return ({
                    async didResolveOperation({ request, document }) {
                        const query = request.operationName
                            ? graphql.separateOperations(document)[request.operationName]
                            : document;
                        const complexity = graphqlQueryComplexity.getComplexity({
                            schema,
                            query,
                            variables: request.variables,
                            estimators,
                        });
                        if (complexity >= maximumComplexity) {
                            throw await createError(maximumComplexity, complexity);
                        }
                        if (onComplete) {
                            await onComplete(complexity);
                        }
                    },
                });
            },
        };
    };

    exports.createComplexityPlugin = createComplexityPlugin;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
